from django.db.models import Manager
from django.utils import timezone


class ResetPasswordManager(Manager):
    def get_valid_token(self, token: str):
        try:
            token = self.get(
                token=token,
                status=self.model.STATUS.PENDING.value,
                expire_date__gte=timezone.now(),
            )
        except self.model.DoesNotExist:
            return None

        return token

    def invalidate_existing_token(self, user):
        return self.filter(user=user, status="pending").update(status="invalid")



class EmailProviderClassNotSet(Exception):
    message = "You need to set EMAIL_PROVIDER_CLASS in your settings"


class EmailProviderClassInvalid(Exception):
    message = (
        "Email provider is invalid please set the route "
        "correctly module.submodule.class"
    )


class RedirectLinkNotSet(Exception):
    message = (
        "You need to set up your redirect link for the default template"
    )


class AppNameNotSet(Exception):
    message = (
        "You need to set up your app name for the default template"
    )



from rest_framework import serializers

from reset_password.models import ResetPasswordToken


class ResetPasswordSerializer(serializers.ModelSerializer):

    class Meta:
        model = ResetPasswordToken
        fields = ("user",)

    def create(self, validated_data):
        return super().create(validated_data)


class PasswordSerializer(serializers.Serializer):
    password = serializers.CharField(max_length=100, required=True)

    def update(self, instance, validated_data):
        raise NotImplementedError()

    def create(self, validated_data):
        raise NotImplementedError()


class TokenQueryParamsSerializer(serializers.Serializer):
    token = serializers.UUIDField(format="hex_verbose")

    def update(self, instance, validated_data):
        raise NotImplementedError()

    def create(self, validated_data):
        raise NotImplementedError()

    def get_token(self):
        return self.initial_data.get("token")



from django.apps import apps
from django.conf import settings
from rest_framework import mixins, viewsets, status
from rest_framework.decorators import action
from rest_framework.response import Response

from reset_password.models import ResetPasswordToken
from reset_password.serializers import (
    ResetPasswordSerializer,
    PasswordSerializer,
    TokenQueryParamsSerializer,
)


class ResetPasswordView(mixins.CreateModelMixin, viewsets.GenericViewSet):
    queryset = ResetPasswordToken.objects.all()
    serializer_class = ResetPasswordSerializer
    permission_classes = []

    def create(self, request, *args, **kwargs):
        module, class_name = settings.AUTH_USER_MODEL.split(".")
        User = apps.get_model(module, class_name)
        user = User.objects.filter(email=request.data["email"]).first()
        if user:
            user = user.id
        serializer = self.get_serializer(data={"user": user})
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)
        headers = self.get_success_headers(serializer.data)
        return Response(status=status.HTTP_201_CREATED, headers=headers)

    @action(methods=["GET"], detail=False, url_path="token-validation")
    def token_validation(self, request):
        token = self.validate_token(self.request.query_params)
        if token is None:
            return Response(status=status.HTTP_400_BAD_REQUEST)

        return Response(status=status.HTTP_200_OK)

    @action(methods=["POST"], detail=False, url_path="submit")
    def change_password(self, request, *args, **kwargs):
        token = self.validate_token(self.request.query_params)

        if token is None:
            return Response(status=status.HTTP_400_BAD_REQUEST)

        PasswordSerializer(data=request.data).is_valid(raise_exception=True)
        token.update_password(request.data.get("password"))

        return Response(status=status.HTTP_200_OK)

    def validate_token(self, query_params):
        serializer = TokenQueryParamsSerializer(data=query_params)
        serializer.is_valid(raise_exception=True)
        token = serializer.get_token()

        return ResetPasswordToken.objects.get_valid_token(token=token)



# Generated by Django 3.0.6 on 2020-07-14 09:37

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='ResetPasswordToken',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('token', models.UUIDField(default=uuid.uuid4)),
                ('expire_date', models.DateTimeField(null=True)),
                ('status', models.CharField(choices=[('accepted', 'accepted'), ('pending', 'pending'), ('invalid', 'invalid')], default='pending', max_length=100)),
                ('created', models.DateTimeField(default=django.utils.timezone.now)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='reset_password', to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))

README = open(os.path.join(here, 'README.rst')).read()

setup(
    name='drf-reset-password',
    version='1.2.1',
    packages=find_packages(exclude=['tests', 'tests.*']),
    include_package_data=True,
    description='Description',
    long_description=README,
    author='linnify',
    zip_safe=False,
    author_email='info@linnify.com',
    url='https://github.com/linnify/drf-reset-password',
    license='MIT',
    install_requires=[
        'Django>=3.0',
        'djangorestframework>=3.11'
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'Framework :: Django',
        'Framework :: Django :: 3.0',
      ],
)


import uuid
from datetime import timedelta
from enum import Enum
from pydoc import locate

import django
from django.db import models
from django.template.loader import render_to_string
from django.utils import timezone

from reset_password.exceptions import (
    EmailProviderClassNotSet,
    EmailProviderClassInvalid,
    RedirectLinkNotSet, AppNameNotSet)
from reset_password.managers import ResetPasswordManager
from django.conf import settings


class StatusType(Enum):
    ACCEPTED = "accepted"
    PENDING = "pending"
    INVALID = "invalid"


STATUS_TYPES = [(e.value, e.value) for e in StatusType]


class ResetPasswordToken(models.Model):
    STATUS = StatusType
    token = models.UUIDField(primary_key=False, default=uuid.uuid4)
    expire_date = models.DateTimeField(null=True)
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        related_name="reset_password",
        on_delete=models.CASCADE,
    )
    status = models.CharField(choices=STATUS_TYPES, default=StatusType.PENDING.value, max_length=100)
    created = models.DateTimeField(default=django.utils.timezone.now)
    objects = ResetPasswordManager()

    def save(self, *args, **kwargs):

        if not self.id:
            ResetPasswordToken.objects.invalidate_existing_token(user=self.user)
            self.expire_date = timezone.now() + timedelta(minutes=self._get_email_expiration_time(),)
            self.send_email()

        return super(ResetPasswordToken, self).save(*args, **kwargs)

    def __str__(self):
        return f"{self.expire_date}-{self.token}-{self.status}"

    def send_email(self):
        mail = self._get_email_provider()
        email = self._get_user_email()

        content = render_to_string(
            self._get_email_template(),
            {"link": self._get_redirect_link()+str(self.token), "app_name": self._get_app_name(), "email": email},
        )
        return mail.send_email(email, self._get_email_title(), content)

    def update_password(self, password: str):
        self.user.set_password(password)
        self.user.save()
        self.status = StatusType.ACCEPTED.value
        self.save()

    def _get_user_email(self):
        if settings.DRF_RESET_EMAIL.get("EMAIL_FIELD"):
            return self.user.__getattribute__(settings.DRF_RESET_EMAIL["EMAIL_FIELD"])
        else:
            return self.user.email

    def _get_email_provider(self):
        if not settings.DRF_RESET_EMAIL.get("EMAIL_PROVIDER"):
            raise EmailProviderClassNotSet()

        email_class = locate(settings.DRF_RESET_EMAIL["EMAIL_PROVIDER"])
        if not email_class:
            raise EmailProviderClassInvalid()
        return email_class()

    def _get_email_template(self):
        return settings.DRF_RESET_EMAIL["RESET_PASSWORD_EMAIL_TEMPLATE"] if settings.DRF_RESET_EMAIL.get("RESET_PASSWORD_EMAIL_TEMPLATE") else "reset_password_email.html"

    def _get_email_title(self):
        return settings.DRF_RESET_EMAIL["RESET_PASSWORD_EMAIL_TITLE"] if settings.DRF_RESET_EMAIL.get("RESET_PASSWORD_EMAIL_TITLE") else "Reset Password"

    def _get_email_expiration_time(self):
        return settings.DRF_RESET_EMAIL["EMAIL_EXPIRATION_TIME"] if settings.DRF_RESET_EMAIL.get("EMAIL_EXPIRATION_TIME") else 24

    def _get_redirect_link(self):
        if not settings.DRF_RESET_EMAIL.get("REDIRECT_LINK"):
            raise RedirectLinkNotSet()
        return settings.DRF_RESET_EMAIL.get("REDIRECT_LINK")

    def _get_app_name(self):
        if not settings.DRF_RESET_EMAIL.get("APP_NAME"):
            raise AppNameNotSet()
        return settings.DRF_RESET_EMAIL.get("APP_NAME")


class EmailProvider:
    def send_email(self, email, title, content):
        if settings.DRF_RESET_EMAIL.get("TEST_ENV"):
            return
        else:
            raise NotImplementedError("You need to implement this method on your email provider")



from django.contrib import admin
from django.urls import reverse
from django.utils.html import format_html

from reset_password.models import ResetPasswordToken


def to_link(field_name, column_name=None, field_show=None):
    def _to_link(obj):
        linked_object = getattr(obj, field_name)
        if linked_object is None:
            return "-"
        app_label = linked_object._meta.app_label
        model_name = linked_object._meta.model_name
        view_name = f"admin:{app_label}_{model_name}_change"
        link_url = reverse(view_name, args=[linked_object.pk])
        display_name = (
            getattr(linked_object, field_show) if field_show else linked_object
        )
        return format_html(f'<a href="{link_url}">{display_name}</a>')

    _to_link.short_description = column_name or field_name
    return _to_link


class ResetPasswordTokenAdmin(admin.ModelAdmin):
    list_display = ("token", "expire_date", "status", to_link("user"))
    ordering = ("token",)
    list_per_page = 20


admin.site.register(ResetPasswordToken, ResetPasswordTokenAdmin)
